==> static_suite-1.0.0-alpha2.tar.gz: static_suite/modules/static_preview/modules/static_preview_subset_build/src/Preview/PreviewManager.php <==
<?php

namespace Drupal\static_preview_subset_build\Preview;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Release\Release;
use Drupal\static_export\Entity\ExportableEntityManagerInterface;
use Drupal\static_export\Semaphore\SemaphoreProvider;
use Drupal\static_suite\StaticSuiteException;
use Throwable;

/**
 * A class to manage previews.
 */
class PreviewManager {

  const PREVIEW_GROUP_FILE_PATH = 'structure/config/preview-group.json';
  const NUMBER_OF_DAYS_TO_KEEP = 7;

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The static builder plugin config.
   *
   * @var array
   */
  protected $pluginConfig;

  /**
   * The data semaphore.
   *
   * @var \Drupal\static_export\Semaphore\Semaphore
   */
  protected $dataSemaphore;

  /**
   * The release manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManager
   */
  protected $releaseManager;

  /**
   * A cache for loaded nodes.
   *
   * @var \Drupal\node\Entity\Node[]
   */
  protected $nodesCache;

  /**
   * Semaphore provider.
   *
   * @var \Drupal\static_export\Semaphore\SemaphoreProvider
   */
  protected $semaphoreProvider;

  /**
   * Exportable entity manager.
   *
   * @var \Drupal\static_export\Entity\ExportableEntityManagerInterface
   */
  protected $exportableEntityManager;

  /**
   * Release constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   Drupal entity manager.
   * @param \Drupal\static_export\Semaphore\SemaphoreProvider $semaphore_provider
   *   Semaphore provider.
   * @param \Drupal\static_export\Entity\ExportableEntityManagerInterface $exportableEntityManager
   *   Exportable entity manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $entity_type_manager, SemaphoreProvider $semaphore_provider, ExportableEntityManagerInterface $exportableEntityManager) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->semaphoreProvider = $semaphore_provider;
    $this->exportableEntityManager = $exportableEntityManager;
  }

  /**
   * Init the preview manager.
   *
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginInterface $staticBuilderPlugin
   *   A Static Builder plugin.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function init(StaticBuilderPluginInterface $staticBuilderPlugin) {
    $this->pluginConfig = $staticBuilderPlugin->getConfiguration();
    $this->dataSemaphore = $this->semaphoreProvider->createDataDirSemaphore();
    $this->releaseManager = $staticBuilderPlugin->getReleaseManager();
  }

  /**
   * Add a node to preview group.
   *
   * Saves data in a JSON file, for easy access from any SSG.
   *
   * @param int $nid
   *   Node id.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function addNodeToPreviewGroup(int $nid) {
    // Acquire a lock.
    $this->dataSemaphore->acquire();

    // Create dir to store data.
    $groupFile = $this->pluginConfig['data-dir'] . "/" . self::PREVIEW_GROUP_FILE_PATH;
    $groupFileDir = dirname($groupFile);
    exec("mkdir -p $groupFileDir", $output, $exitCode);
    if ($exitCode !== 0) {
      throw new StaticSuiteException("Unable to create dir $groupFileDir:\n" . implode("\n", $output));
    }

    // Clear old data.
    $groupData = @file_get_contents($groupFile);
    if ($groupData) {
      $groupData = json_decode($groupData, TRUE);
      $validItems = $this->removeOldItems($groupData);
    }
    else {
      $validItems = [];
    }

    // Add new data.
    $validItems[$nid] = time();
    asort($validItems, SORT_NUMERIC);

    // Save data in an atomic operation.
    $groupFileTemp = $groupFile . '.temp';
    $result = file_put_contents($groupFileTemp, json_encode($validItems, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT));
    if ($result === FALSE) {
      throw new StaticSuiteException("Error while writing to file: $groupFileTemp");
    }
    $result = rename($groupFileTemp, $groupFile);
    if ($result === FALSE) {
      throw new StaticSuiteException("Error while renaming file $groupFileTemp to $groupFile");
    }

    // Release the lock.
    $this->dataSemaphore->release();

    return TRUE;
  }

  /**
   * Remove old preview items from an array of items.
   *
   * @param array $items
   *   An array of preview items.
   *
   * @return array
   *   The array with old items removed.
   */
  protected function removeOldItems(array $items): array {
    $validItems = [];
    $timeLimitToKeep = time() - 60 * 60 * 24 * self::NUMBER_OF_DAYS_TO_KEEP;
    foreach ($items as $nid => $timestamp) {
      // Remove old items.
      if ($timestamp >= $timeLimitToKeep) {
        $validItems[$nid] = $timestamp;
      }
    }
    return $validItems;
  }

  /**
   * Tells whether a node can be previewed.
   *
   * @param int $nid
   *   Node id to be previewed.
   *
   * @return bool
   *   True if it can be previewed.
   */
  public function canBePreviewed(int $nid): bool {
    $canBePreviewed = FALSE;
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      ;
      if ($node instanceof Node &&
        !$node->isPublished() &&
        ($exportableEntity = $this->exportableEntityManager->getExportableEntity($node)) &&
        $exportableEntity->getIsStatifiedPage()) {
        $canBePreviewed = TRUE;
      }
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $canBePreviewed;
  }

  /**
   * Get a node's preview url, prefixed with the preview path.
   *
   * @param int $nid
   *   Node id.
   *
   * @return string
   *   Node url.
   */
  public function getNodeUrl(int $nid) {
    $options = ['absolute' => FALSE];
    $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], $options);
    $url = $url->toString();
    $config = $this->configFactory->get('static_preview_subset_build.settings');
    return $config->get('preview_path') . $url;
  }

  /**
   * Get a node's title.
   *
   * @param int $nid
   *   Node id.
   *
   * @return string
   *   Node title.
   */
  public function getNodeTitle(int $nid) {
    $title = "";
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      $title = $node->label();
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $title;
  }

  /**
   * Get a node's modification time in seconds.
   *
   * @param int $nid
   *   Node id.
   *
   * @return int
   *   Seconds since Epoch.
   */
  public function getNodeChangedTime(int $nid) {
    $changedTime = 0;
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      $changedTime = $node->getChangedTime();
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $changedTime;
  }

  /**
   * Tell whether this node's preview is stale.
   *
   * @param int $nid
   *   Node id.
   *
   * @return bool
   *   True if preview is stale.
   */
  public function isNodePreviewStale(int $nid) {
    $isStale = FALSE;
    // Get current releases preview-group file.
    $currentRelease = $this->releaseManager->getCurrentRelease();
    $previewGroup = [];
    if ($currentRelease !== NULL) {
      $previewGroupFile = $currentRelease->getDir() . "/" . Release::METADATA_DIR . "/" . basename(self::PREVIEW_GROUP_FILE_PATH);
      if (is_file($previewGroupFile) && is_readable($previewGroupFile)) {
        $previewGroup = json_decode(file_get_contents($previewGroupFile), TRUE);
      }
    }

    // If node's change date is newer than the one that appears in preview
    // group, then that preview is stale.
    if (empty($previewGroup[$nid]) || $previewGroup[$nid] < $this->getNodeChangedTime($nid)) {
      return TRUE;
    }

    return $isStale;
  }

}

==> static_suite-1.1.0-alpha2.tar.gz: static_suite/modules/static_preview/modules/static_preview_subset_build/src/Preview/PreviewManager.php <==
<?php

namespace Drupal\static_preview_subset_build\Preview;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\static_build\Plugin\StaticBuilderPluginInterface;
use Drupal\static_build\Release\Release;
use Drupal\static_export\Entity\ExportableEntityManagerInterface;
use Drupal\static_export\Exporter\ExporterPluginInterface;
use Drupal\static_suite\Lock\LockBackendInterface;
use Drupal\static_suite\StaticSuiteException;
use Throwable;

/**
 * A class to manage previews.
 */
class PreviewManager {

  const PREVIEW_GROUP_FILE_PATH = 'structure/config/preview-group.json';
  const NUMBER_OF_DAYS_TO_KEEP = 7;

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The static builder plugin config.
   *
   * @var array
   */
  protected $pluginConfig;

  /**
   * The release manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManager
   */
  protected $releaseManager;

  /**
   * A cache for loaded nodes.
   *
   * @var \Drupal\node\Entity\Node[]
   */
  protected $nodesCache;

  /**
   * The lock system from Static Suite.
   *
   * @var \Drupal\Core\Lock\LockBackendInterface
   */
  protected $lock;

  /**
   * Exportable entity manager.
   *
   * @var \Drupal\static_export\Entity\ExportableEntityManagerInterface
   */
  protected $exportableEntityManager;

  /**
   * Release constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   Drupal entity manager.
   * @param \Drupal\static_suite\Lock\LockBackendInterface $lock
   *   The lock system from Static Suite.
   * @param \Drupal\static_export\Entity\ExportableEntityManagerInterface $exportableEntityManager
   *   Exportable entity manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $entity_type_manager, LockBackendInterface $lock, ExportableEntityManagerInterface $exportableEntityManager) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->lock = $lock;
    $this->exportableEntityManager = $exportableEntityManager;
  }

  /**
   * Init the preview manager.
   *
   * @param \Drupal\static_build\Plugin\StaticBuilderPluginInterface $staticBuilderPlugin
   *   A Static Builder plugin.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function init(StaticBuilderPluginInterface $staticBuilderPlugin) {
    $this->pluginConfig = $staticBuilderPlugin->getConfiguration();
    $this->releaseManager = $staticBuilderPlugin->getReleaseManager();
  }

  /**
   * Add a node to preview group.
   *
   * Saves data in a JSON file, for easy access from any SSG.
   *
   * @param int $nid
   *   Node id.
   *
   * @return bool
   *   True on success.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function addNodeToPreviewGroup(int $nid) {
    // Acquire a lock.
    if (!$this->lock->acquireOrWait(ExporterPluginInterface::DATA_DIR_LOCK_NAME)) {
      throw new StaticSuiteException('Could not acquire lock on "' . $this->pluginConfig['data-dir'] . '", timeout reached.');
    }

    // Create dir to store data.
    $groupFile = $this->pluginConfig['data-dir'] . "/" . self::PREVIEW_GROUP_FILE_PATH;
    $groupFileDir = dirname($groupFile);
    exec("mkdir -p $groupFileDir", $output, $exitCode);
    if ($exitCode !== 0) {
      throw new StaticSuiteException("Unable to create dir $groupFileDir:\n" . implode("\n", $output));
    }

    // Clear old data.
    $groupData = @file_get_contents($groupFile);
    if ($groupData) {
      $groupData = json_decode($groupData, TRUE);
      $validItems = $this->removeOldItems($groupData);
    }
    else {
      $validItems = [];
    }

    // Add new data.
    $validItems[$nid] = time();
    asort($validItems, SORT_NUMERIC);

    // Save data in an atomic operation.
    $groupFileTemp = $groupFile . '.temp';
    $result = file_put_contents($groupFileTemp, json_encode($validItems, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT));
    if ($result === FALSE) {
      throw new StaticSuiteException("Error while writing to file: $groupFileTemp");
    }
    $result = rename($groupFileTemp, $groupFile);
    if ($result === FALSE) {
      throw new StaticSuiteException("Error while renaming file $groupFileTemp to $groupFile");
    }

    // Release the lock.
    $this->lock->release(ExporterPluginInterface::DATA_DIR_LOCK_NAME);

    return TRUE;
  }

  /**
   * Remove old preview items from an array of items.
   *
   * @param array $items
   *   An array of preview items.
   *
   * @return array
   *   The array with old items removed.
   */
  protected function removeOldItems(array $items): array {
    $validItems = [];
    $timeLimitToKeep = time() - 60 * 60 * 24 * self::NUMBER_OF_DAYS_TO_KEEP;
    foreach ($items as $nid => $timestamp) {
      // Remove old items.
      if ($timestamp >= $timeLimitToKeep) {
        $validItems[$nid] = $timestamp;
      }
    }
    return $validItems;
  }

  /**
   * Tells whether a node can be previewed.
   *
   * @param int $nid
   *   Node id to be previewed.
   *
   * @return bool
   *   True if it can be previewed.
   */
  public function canBePreviewed(int $nid): bool {
    $canBePreviewed = FALSE;
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      if ($node instanceof Node &&
        !$node->isPublished() &&
        ($exportableEntity = $this->exportableEntityManager->getExportableEntity($node)) &&
        $exportableEntity->getIsStatifiedPage()) {
        $canBePreviewed = TRUE;
      }
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $canBePreviewed;
  }

  /**
   * Get a node's preview url, prefixed with the preview path.
   *
   * @param int $nid
   *   Node id.
   *
   * @return string
   *   Node url.
   */
  public function getNodeUrl(int $nid) {
    $options = ['absolute' => FALSE];
    $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], $options);
    $url = $url->toString();
    $config = $this->configFactory->get('static_preview_subset_build.settings');
    return $config->get('preview_path') . $url;
  }

  /**
   * Get a node's title.
   *
   * @param int $nid
   *   Node id.
   *
   * @return string
   *   Node title.
   */
  public function getNodeTitle(int $nid) {
    $title = "";
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      $title = $node->label();
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $title;
  }

  /**
   * Get a node's modification time in seconds.
   *
   * @param int $nid
   *   Node id.
   *
   * @return int
   *   Seconds since Epoch.
   */
  public function getNodeChangedTime(int $nid) {
    $changedTime = 0;
    try {
      if (empty($this->nodesCache[$nid])) {
        $storage = $this->entityTypeManager->getStorage("node");
        $this->nodesCache[$nid] = $storage->load($nid);
      }
      $node = $this->nodesCache[$nid];
      $changedTime = $node->getChangedTime();
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $changedTime;
  }

  /**
   * Tell whether this node's preview is stale.
   *
   * @param int $nid
   *   Node id.
   *
   * @return bool
   *   True if preview is stale.
   */
  public function isNodePreviewStale(int $nid) {
    $isStale = FALSE;
    // Get current releases preview-group file.
    $currentRelease = $this->releaseManager->getCurrentRelease();
    $previewGroup = [];
    if ($currentRelease !== NULL) {
      $previewGroupFile = $currentRelease->getDir() . "/" . Release::METADATA_DIR . "/" . basename(self::PREVIEW_GROUP_FILE_PATH);
      if (is_file($previewGroupFile) && is_readable($previewGroupFile)) {
        $previewGroup = json_decode(file_get_contents($previewGroupFile), TRUE);
      }
    }

    // If node's change date is newer than the one that appears in preview
    // group, then that preview is stale.
    if (empty($previewGroup[$nid]) || $previewGroup[$nid] < $this->getNodeChangedTime($nid)) {
      return TRUE;
    }

    return $isStale;
  }

}
